
#version 310 es
#pragma only_renderers glsl31 metal
#pragma input cs_in
#pragma end

#pragma output cs_out
#pragma end

#pragma cs cs_in cs_out
precision highp float;
precision mediump sampler2D;
precision highp sampler3D;

struct BuiltinParameters
{
    float delta;
    uint dispatchWidth;
    uint nbSpawned;
    uint spawnId;
    uint systemSeed;
};
    
layout(binding = 0, std140) uniform builtiParameters
{
    BuiltinParameters builtiParametersValue;
};

struct deadListIn_type {
    uint[1] value;
};

layout(std430, binding = 0) buffer attributeBuffer {
    uint attributeBuffer_buf[];
};
layout(std430, binding = 1) buffer deadListIn {
    uint deadList_andCounter_Buf[];
};

layout(std430, binding = 2) buffer eventListOut0 {
    uint eventListOut0_buf[];
};
layout(std430, binding = 3) buffer eventListOut_Counter0 {
    uint eventListOut_Counter0_Buf[];
};

#define VFX_PI 3.1415926538
#define VFX_PI2 6.28318530718
#define VFX_M_PI_F 3.14159265359
#define VFX_HALF_PI 1.57079632679
#define VFX_ONE_THIRD_PI 1.0471975512
#define VFX_FLT_MIN 1.175494351e-38
#define VFX_EPSILON 1e-5
uint wang_hash(uint seed)
{
    seed = (seed ^ 61u) ^ (seed >> 16u);
    seed *= 9u;
    seed = seed ^ (seed >> 4u);
    seed *= 0x27d4eb2du;
    seed = seed ^ (seed >> 15u);
    return seed;
}

uint AnotherHash(uint seed)
{
    seed = ((seed >> 16u) ^ seed) * 0x45d9f3bu;
    seed = ((seed >> 16u) ^ seed) * 0x45d9f3bu;
    seed = (seed >> 16u) ^ seed;
    return seed;
}

float Rand(inout uint seed)
{
    seed = 0x0019660du * seed + 0x3c6ef35fu;
    float t = uintBitsToFloat((seed >> 9u) | 0x3f800000u);
    return t - 1.0f;
}
    
vec2 Rand2(inout uint seed)
{
    return vec2(Rand(seed), Rand(seed));
}

vec3 Rand3(inout uint seed)
{
    return vec3(Rand(seed), Rand(seed), Rand(seed));
}

float random(inout uint seed, float A, float B)
{
    float rand = Rand(seed);
    return mix(A, B, rand);
}

vec3 random3(inout uint seed, vec3 A, vec3 B)
{
    vec3 rand3 = Rand3(seed);
    return mix(A, B, rand3);
}

vec3 VFXSafeNormalize(vec3 v)
{
    float sqrLength = max(VFX_FLT_MIN,dot(v,v));
    return v * inversesqrt(sqrLength);
}

vec3 HUEtoRGB(float H)
{
    float R = abs(H * 6.0 - 3.0) - 1.0;
    float G = 2.0 - abs(H * 6.0 - 2.0);
    float B = 2.0 - abs(H * 6.0 - 4.0);
    return clamp(vec3(R,G,B), 0.0, 1.0);
}

vec3 RGBtoHCV(vec3 RGB)
{
    vec4 P = (RGB.g < RGB.b) ? vec4(RGB.bg, -1.0, 2.0/3.0) : vec4(RGB.gb, 0.0, -1.0/3.0);
    vec4 Q = (RGB.r < P.x) ? vec4(P.xyw, RGB.r) : vec4(RGB.r, P.yzx);
    float C = Q.x - min(Q.w, Q.y);
    float H = abs((Q.w - Q.y) / (6.0 * C + 1e-10) + Q.z);
    return vec3(H, C, Q.x);
}

vec3 RGBtoHSV(vec3 RGB)
{
    vec3 HCV = RGBtoHCV(RGB);
    float S = HCV.y / (HCV.z + 1e-10);
    return vec3(HCV.x, S, HCV.z);
}

vec3 HSVtoRGB(vec3 HSV)
{
    return ((HUEtoRGB(HSV.x) - 1.0) * HSV.y + 1.0) * HSV.z;
}

vec3 quintic(vec3 x)
{
    return x * x * x * (x * (x * 6.0 - 15.0) + 10.0);
}

float hash(vec3 v)
{
    float HASHSCALE = 0.1031;
    v = fract(v * HASHSCALE);
    v += dot(v, v.yzx + 19.19);
    return fract((v.x + v.y) * v.z);
}

float noise01(float x, float y, float z)
{
    // Gradients
    vec3 PerlinNoise_Gradients[12];
    PerlinNoise_Gradients[0] = vec3(1, 1, 0);
    PerlinNoise_Gradients[1] = vec3(-1, 1, 0);
    PerlinNoise_Gradients[2] = vec3(1, -1, 0);
    PerlinNoise_Gradients[3] = vec3(-1, -1, 0);
    PerlinNoise_Gradients[4] = vec3(1, 0, 1);
    PerlinNoise_Gradients[6] = vec3(1, 0, -1);
    PerlinNoise_Gradients[7] = vec3(-1, 0, -1);
    PerlinNoise_Gradients[8] = vec3(0, 1, 1);
    PerlinNoise_Gradients[9] = vec3(0, -1, 1);
    PerlinNoise_Gradients[10] = vec3(0, 1, -1);
    PerlinNoise_Gradients[11] = vec3(0, -1, -1);

    vec3 p = floor(vec3(x, y, z));
    vec3 t = vec3(x, y, z) - p;
    vec3 f = quintic(t);
    vec3 v0 = p + vec3(0.0, 0.0, 0.0);
    vec3 v1 = p + vec3(1.0, 0.0, 0.0);
    vec3 v2 = p + vec3(0.0, 1.0, 0.0);
    vec3 v3 = p + vec3(1.0, 1.0, 0.0);
    vec3 v4 = v0 + vec3(0.0, 0.0, 1.0);
    vec3 v5 = v1 + vec3(0.0, 0.0, 1.0);
    vec3 v6 = v2 + vec3(0.0, 0.0, 1.0);
    vec3 v7 = v3 + vec3(0.0, 0.0, 1.0);

    int g0Index = int(hash(v0) * 12.0);
    int g1Index = int(hash(v1) * 12.0);
    int g2Index = int(hash(v2) * 12.0);
    int g3Index = int(hash(v3) * 12.0);
    int g4Index = int(hash(v4) * 12.0);
    int g5Index = int(hash(v5) * 12.0);
    int g6Index = int(hash(v6) * 12.0);
    int g7Index = int(hash(v7) * 12.0);
    vec3 g0 = PerlinNoise_Gradients[g0Index];
    vec3 g1 = PerlinNoise_Gradients[g1Index];
    vec3 g2 = PerlinNoise_Gradients[g2Index];
    vec3 g3 = PerlinNoise_Gradients[g3Index];
    vec3 g4 = PerlinNoise_Gradients[g4Index];
    vec3 g5 = PerlinNoise_Gradients[g5Index];
    vec3 g6 = PerlinNoise_Gradients[g6Index];
    vec3 g7 = PerlinNoise_Gradients[g7Index];

    return mix(
        mix(
            mix(dot(g0, t - vec3(0.0, 0.0, 0.0)), dot(g1, t - vec3(1.0, 0.0, 0.0)), f.x),
            mix(dot(g2, t - vec3(0.0, 1.0, 0.0)), dot(g3, t - vec3(1.0, 1.0, 0.0)), f.x),
            f.y
        ),
        mix(
            mix(dot(g4, t - vec3(0.0, 0.0, 1.0)), dot(g5, t - vec3(1.0, 0.0, 1.0)), f.x),
            mix(dot(g6, t - vec3(0.0, 1.0, 1.0)), dot(g7, t - vec3(1.0, 1.0, 1.0)), f.x),
            f.y
        ),
        f.z
    );
}

float fBm(int octaves, float x, float y, float z, float Fre)
{
    float amplitude = 1.0;
    float frequency = Fre;
    float result = 0.0;
    float totalAmplitude = 0.0;
    
    for (int i = 0; i < octaves; i++) {
        vec3 pos = frequency * vec3(x, y, z);
        result = result + amplitude * noise01(pos.x, pos.y, pos.z);
        totalAmplitude = totalAmplitude + amplitude;
        amplitude = amplitude / 2.0;
        frequency = frequency * 2.0;
    }
    result = result / totalAmplitude;
    return result;
}

vec3 curlNoise(vec3 oldPos, float NoiseScale, float Frequency, int Octaves)
{
    float ParticleZoneSize = 5.0;
    float halfZone = ParticleZoneSize / 2.0;
    // Get noise value
    float eps = 0.0001;
    vec3 uvy = (oldPos + vec3(halfZone, halfZone, halfZone)) / ParticleZoneSize;
    uvy = uvy * NoiseScale;

    vec3 curl = vec3(0.0, 0.0, 0.0);
    float n = 0.0, a = 0.0, b = 0.0;
    float o = fBm(Octaves, uvy.x, uvy.y, uvy.z, Frequency);

    n = fBm(Octaves, uvy.x, uvy.y - eps, uvy.z, Frequency);
    a = (o - n) / eps;
    n = fBm(Octaves, uvy.x, uvy.y, uvy.z - eps, Frequency);
    b = (o - n) / eps;
    curl.x = a - b;

    n = fBm(Octaves, uvy.x, uvy.y, uvy.z - eps, Frequency);
    a = (o - n) / eps;
    n = fBm(Octaves, uvy.x - eps, uvy.y, uvy.z, Frequency);
    b = (o - n) / eps;
    curl.y = a - b;

    n = fBm(Octaves, uvy.x - eps, uvy.y, uvy.z, Frequency);
    a = (o - n) / eps;
    n = fBm(Octaves, uvy.x, uvy.y - eps, uvy.z, Frequency);
    b = (o - n) / eps;
    curl.z = a - b;

    return curl * NoiseScale * 0.01;
}

void Orient_4(inout vec3 axisX, inout vec3 axisY, inout vec3 axisZ, mat4 viewRot) /*mode:FaceCameraPlane axes:ZY */
{
    axisX = vec3(viewRot[0].x, viewRot[1].x, viewRot[2].x);
    axisY = vec3(viewRot[0].y, viewRot[1].y, viewRot[2].y);
    axisZ = vec3(viewRot[0].z, viewRot[1].z, viewRot[2].z);
}

mat3 GetScaleMatrix(vec3 scale)
{
    return mat3(scale.x,    0.0,          0.0,
                    0.0,          scale.y,    0.0,
                    0.0,          0.0,          scale.z);
}

mat3 GetEulerMatrix(vec3 angles)
{
    vec3 s,c;
    s = sin(angles);
    c = cos(angles);

    return transpose(mat3(c.y * c.z + s.x * s.y * s.z,    c.z * s.x * s.y - c.y * s.z,    c.x * s.y,
                    c.x * s.z,                      c.x * c.z,                      -s.x,
                    -c.z * s.y + c.y * s.x * s.z,   c.y * c.z * s.x + s.y * s.z,    c.x * c.y));
}
    
mat4 GetElementToVFXMatrix(vec3 axisX,vec3 axisY,vec3 axisZ,mat3 rot,vec3 pivot,vec3 size,vec3 pos)
{
    mat3 rotAndScale = GetScaleMatrix(size);
    rotAndScale = rotAndScale * rot;
    rotAndScale = rotAndScale * transpose(mat3(axisX,axisY,axisZ));
    pos -= (pivot * rotAndScale);
    return transpose(mat4(
        vec4(rotAndScale[0],pos.x),
        vec4(rotAndScale[1],pos.y),
        vec4(rotAndScale[2],pos.z),
        vec4(0.0,0.0,0.0,1.0)));
}

mat4 GetElementToVFXMatrix(vec3 axisX,vec3 axisY,vec3 axisZ,vec3 angles,vec3 pivot,vec3 size,vec3 pos)
{
    mat3 rot = GetEulerMatrix(angles);
    return GetElementToVFXMatrix(axisX,axisY,axisZ,rot,pivot,size,pos);
}

vec2 ParticleScreenUV(vec3 position, vec3 size, vec3 pivot, vec3 angle, uint inWorldSpace, mat4 model, mat4 view, mat4 projection) {
  vec3 worldPos = (model * vec4(position, 1.0)).xyz;
  vec3 axisX = vec3(1.0, 0.0, 0.0);
  vec3 axisY = vec3(0.0, 1.0, 0.0);
  vec3 axisZ = vec3(0.0, 0.0, 1.0);

  axisX = vec3(view[0].x, view[1].x, view[2].x);
  axisY = vec3(view[0].y, view[1].y, view[2].y);
  axisZ = vec3(view[0].z, view[1].z, view[2].z);
  // Orient_4(axisX, axisY, axisZ, view);
  mat4 elementToVFX = GetElementToVFXMatrix(
                      axisX,
                      axisY,
                      axisZ,
                      angle,
                      pivot,
                      size,
                      position);
  vec3 posInWorld = position;
  if(inWorldSpace == 0u){
      posInWorld = (model * elementToVFX * vec4(0.0, 0.0, 0.0, 1.0)).xyz;
  } else {
      posInWorld = (elementToVFX * vec4(0.0, 0.0, 0.0,    1.0)).xyz;
  }
  vec4 clipPos = projection * view * vec4(posInWorld, 1.0);
  vec3 ndcPos = clipPos.xyz / clipPos.w;
  vec2 screenUV = (ndcPos.xy + vec2(1.0)) * 0.5;
  return screenUV;
}

vec3 radians3(vec3 degree)
{
    return vec3(radians(degree.x), radians(degree.y), radians(degree.z));
}


mat4 GetModelMatrix(vec3 eulerRotation, vec3 translation, float flag)
{
    vec3 s, c;
    s = sin(eulerRotation);
    c = cos(eulerRotation);
    mat4 Rx;
    Rx[0] = vec4(1.0, 0.0, 0.0, 0.0);
    Rx[1] = vec4(0.0, c.x, s.x, 0.0);
    Rx[2] = vec4(0.0, -s.x, c.x, 0.0);
    Rx[3] = vec4(0.0, 0.0, 0.0, 1.0);
    mat4 Ry;
    Ry[0] = vec4(c.y, 0.0, s.y, 0.0);
    Ry[1] = vec4(0.0, 1.0, 0.0, 0.0);
    Ry[2] = vec4(-s.y, 0.0, c.y, 0.0);
    Ry[3] = vec4(0.0, 0.0, 0.0, 1.0);
    mat4 Rz;
    Rz[0] = vec4(c.z, s.z, 0.0, 0.0);
    Rz[1] = vec4(-s.z, c.z, 0.0, 0.0);
    Rz[2] = vec4(0.0, 0.0, 1.0, 0.0);
    Rz[3] = vec4(0.0, 0.0, 0.0, 1.0);
    mat4 Rw;
    Rw[0] = vec4(1.0, 0.0, 0.0, 0.0);
    Rw[1] = vec4(0.0, 1.0, 0.0, 0.0);
    Rw[2] = vec4(0.0, 0.0, 1.0, 0.0);
    Rw[3] = vec4(-translation.x, -translation.y, -translation.z, 1.0);
    if (flag <= 0.0) return Rw * Rx * Ry * Rz;
    return Rz * Ry * Rx * Rw;
}

mat4 GetTransformInvMatrix(vec3 angles,vec3 scale,vec3 pos)
{
    mat3 invrotAndScale = GetScaleMatrix(1.0/scale);
    mat3 invrot = GetEulerMatrix(-angles);
    invrotAndScale = invrot * invrotAndScale;
    mat4 inv = mat4(
        vec4(invrotAndScale[0],0),
        vec4(invrotAndScale[1],0),
        vec4(invrotAndScale[2],0),
        vec4(0,0,0,1));
    mat4 invpos = mat4(
        vec4(1.0,0,0,-pos.x),
        vec4(0,1.0,0,-pos.y),
        vec4(0,0,1,-pos.z),
        vec4(0,0,0,1));
    return transpose(invpos * inv);
}
    
mat4 GetTransformMatrix(vec3 angles,vec3 scale,vec3 pos)
{
    mat3 rotAndScale = GetScaleMatrix(scale);
    mat3 rot = GetEulerMatrix(angles);
    rotAndScale = rotAndScale * rot;
    return transpose(mat4(
        vec4(rotAndScale[0],pos.x),
        vec4(rotAndScale[1],pos.y),
        vec4(rotAndScale[2],pos.z),
        vec4(0,0,0,1)));
}
  
void Age(inout float age, float deltaTime)
{
    age += deltaTime;
}

uint Reap(float age, float lifetime)
{
    if(age > lifetime) { return 0u; }
    return 1u;
}

vec3 RaySphereIntersection(vec3 O, vec3 D, vec3 C, float R)
{
    vec3 L = C - O;
    float DT = dot(L, D);
    float R2 = R * R;
    float CT2 = dot(L, L) - DT * DT;
    if (CT2 > R2) return vec3(0.0, 0.0, 0.0);
    float AT = sqrt(R2 - CT2);
    float BT = AT;
    float OA = DT - AT;
    float OB = DT + BT;
    return vec3(OA, OB, 1);
}

float sdBox(vec3 pos, vec3 halfSize)
{
    vec3 q = abs(pos) - halfSize;
    return length(max(q, 0.0)) + min(max(q.x, max(q.y, q.z)), 0.0);
}

float GetDistance_Point2LineSegment(vec3 pt, vec3 p1, vec3 p2)
{
    if (dot(pt - p1, p2 - p1) < 0.0) return distance(pt, p1);
    if (dot(pt - p2, p1 - p2) < 0.0) return distance(pt, p2);
    float CosAlpha = dot(pt - p1, p2 - p1) / length(pt - p1) / length(p2 - p1);
    float SinAlpha = sqrt(1.0 - CosAlpha * CosAlpha);
    return distance(pt, p1) * SinAlpha;
}
    
float mySign(float v)
{
    int a = int((0.0<v)?0xFFFFFFFFu: uint(0));
    int b = int((v<0.0)?0xFFFFFFFFu: uint(0));
    float res = float(-a+b);
    return res;
}
    
vec3 SampleSDFDerivativesFast(sampler3D Map, vec3 coord, float dist)
{
    vec3 dir = vec3(0);
    vec3 xDeviation = coord + vec3(0.01, 0.0, 0.0);
    vec3 yDeviation = coord + vec3(0.0, 0.01, 0.0);
    vec3 zDeviation = coord + vec3(0.0, 0.0, 0.01);
    
    dir.x = texture(Map, xDeviation).x;
    dir.y = texture(Map, yDeviation).x;
    dir.z = texture(Map, zDeviation).x;

    dir = dir * 1.0;
    dir = (-vec3(dist)) + dir;
    return dir;
}


float fma2(float a, float b, float c)
{
    return a * b +c;
}
    
vec4 NoiseHash2D(vec2 gridcell)
{
    vec2 kOffset = vec2(26.0, 161.0);
    float kDomain = 71.0;
    float kLargeFloat = 1.0 / 951.135664f;

    vec4 P = vec4(gridcell.xy, gridcell.xy + 1.0);
    P = P - floor(P * (1.0 / kDomain)) * kDomain;  // truncate the domain
    P += kOffset.xyxy;                             // offset to interesting part of the noise
    P *= P;                                        // calculate and return the hash
    return fract(P.xzxz * P.yyww * kLargeFloat);
}

vec4 Interpolation_InterpAndDeriv(vec2 x)
{
    return (x.xyxy * x.xyxy * (x.xyxy * (x.xyxy * (x.xyxy * vec2(6.0, 0.0).xxyy + vec2(-15.0, 30.0).xxyy) + vec2(10.0, -60.0).xxyy) + vec2(0.0, 30.0).xxyy));
}

vec3 GenerateValueNoise2D(vec2 coordinate)
{
    vec2 i = floor(coordinate);
    vec2 f = coordinate - i;

    vec4 hash = NoiseHash2D(i);

    vec4 blend = Interpolation_InterpAndDeriv(f);

    vec4 res0 = mix(hash.xyxz, hash.zwyw, blend.yyxx);
    vec2 resDelta = res0.yw - res0.xz;

    float noise = res0.x + resDelta.x * blend.x;
    vec2 deriv = resDelta * blend.zw;
    return vec3(noise, deriv);
}

vec3 TurbulanceAux(vec3 coordinate, float frequency, int octaveCount, float persistence, float lacunarity)
{
    vec2 total[3];
    total[0] = vec2(0.0, 0.0);
    total[1] = vec2(0.0, 0.0);
    total[2] = vec2(0.0, 0.0);

    float amplitude = 1.0;
    float totalAmplitude = 0.0;

    vec2 points[3];
    points[0] = coordinate.zy;
    points[1] = coordinate.xz + 100.0;
    points[2] = coordinate.yx + 200.0;

    for (int octaveIndex = 0; octaveIndex < octaveCount; octaveIndex++)
    {
        for (int i = 0; i < 3; i++)
        {
            vec2 derivatives = GenerateValueNoise2D(points[i] * frequency).yz;

            total[i] += derivatives * amplitude;
        }

        totalAmplitude += amplitude;
        amplitude *= persistence;
        frequency *= lacunarity;
    }

    return vec3(
        (total[2].x - total[1].y),
        (total[0].x - total[2].y),
        (total[1].x - total[0].y)) / totalAmplitude;
}

layout(local_size_x = 64, local_size_y = 1, local_size_z = 1) in;
  uniform sampler2D Gradient;
struct ExternalParameters
{
    vec4 Position_1;
    vec4 Vector3f_0;
    float Alpha_1;
    float Float_0;
    float Float_1;
    float Float_2;
    float Float_3;
    float Rate;
    bool Alive;
    bool Boolean_0;
};

layout(binding = 1, std140) uniform externalParameters {
    ExternalParameters externalParametersValue;
};

// float Curve(float t)
// {
//     t = clamp(t, 0.001, 0.999);
//     vec4 p = texture(DefaultCurveTexture, vec2(t, 0.0)).abgr;
//     // Decode rgba to float
//     float v = dot( p, vec4(1.0, 1.0/255.0, 1.0/65025.0, 1.0/16581375.0) );
//     v = v * 3.0 - 1.0;
//     return v;
// }
// vec4 Gradient(float t)
// {
//     t = clamp(t, 0.001, 0.999);
//     vec4 p = texture(DefaultGradientTexture, vec2(t, 0.0));
//     return p;
// }
    mat4 rotationMatrix(vec3 axis, float angle) {
    axis = normalize(axis);
    float s = sin(angle);
    float c = cos(angle);
    float oc = 1.0 - c;
    
    return mat4(oc * axis.x * axis.x + c,           oc * axis.x * axis.y - axis.z * s,  oc * axis.z * axis.x + axis.y * s,  0.0,
                oc * axis.x * axis.y + axis.z * s,  oc * axis.y * axis.y + c,           oc * axis.y * axis.z - axis.x * s,  0.0,
                oc * axis.z * axis.x - axis.y * s,  oc * axis.y * axis.z + axis.x * s,  oc * axis.z * axis.z + c,           0.0,
                0.0,                                0.0,                                0.0,                                1.0);
}

vec3 Rotate3D(vec3 v, vec3 center, vec3 axis, float angle) {
	mat4 m = rotationMatrix(axis, angle);
	return (m * vec4(v - center, 1.0)).xyz + center;
}
void SetPosition_1(inout vec3 position, vec4 Position)
{
position.xyz = Position.xyz;
}

void SetAlive(inout uint alive, bool Alive)
{
alive = uint(Alive?1:0);
}

void SetAlpha_1(inout float alpha, float Alpha)
{
alpha = Alpha;
}

void TriggerEventRate(inout float rateCount, inout uint eventCount, inout vec3 velocity, float Rate, float deltaTime)
{
rateCount += deltaTime * float(Rate);
uint count = uint(floor(rateCount));
rateCount = fract(rateCount);
eventCount = count;
}


#pragma end
void compute()
{
    uint particleIdx = gl_GlobalInvocationID.x;
    uint countLeft = deadList_andCounter_Buf[0];
    uint maxThreadId = builtiParametersValue.nbSpawned;
    //maxThreadId = min(maxThreadId, countLeft);

    // add for gpu event
    uint eventCount = 0u;
    if(particleIdx < uint(maxThreadId))
    {
        uint index_c = particleIdx << 6;
        uint alive = attributeBuffer_buf[index_c + 12u];
        if (alive != uint(0))
        {
        vec2 uv = vec2(0.0);
      vec3 position = vec3(0.0,0.0,0.0);
      position.x = uintBitsToFloat(attributeBuffer_buf[index_c]);
      position.y = uintBitsToFloat(attributeBuffer_buf[index_c + 1u]);
      position.z = uintBitsToFloat(attributeBuffer_buf[index_c + 2u]);
      vec3 velocity = vec3(0.0,0.0,0.0);
      velocity.x = uintBitsToFloat(attributeBuffer_buf[index_c + 4u]);
      velocity.y = uintBitsToFloat(attributeBuffer_buf[index_c + 5u]);
      velocity.z = uintBitsToFloat(attributeBuffer_buf[index_c + 6u]);
      vec3 direction = vec3(0.0,0.0,0.0);
      direction.x = uintBitsToFloat(attributeBuffer_buf[index_c + 7u]);
      direction.y = uintBitsToFloat(attributeBuffer_buf[index_c + 8u]);
      direction.z = uintBitsToFloat(attributeBuffer_buf[index_c + 9u]);
      float lifetime = uintBitsToFloat(attributeBuffer_buf[index_c + 10u]);
      float age = uintBitsToFloat(attributeBuffer_buf[index_c + 11u]);
      uint inWorldSpace = attributeBuffer_buf[index_c + 44u];
      uint originalParticleIdx = attributeBuffer_buf[index_c + 45u];
    float alpha = uintBitsToFloat(attributeBuffer_buf[index_c + 17u]);
        float rateCount = uintBitsToFloat(attributeBuffer_buf[index_c + 26u]);
        vec3 targetPosition = vec3(0.0,0.0,0.0);
        targetPosition.x = uintBitsToFloat(attributeBuffer_buf[index_c + 19u]);
        targetPosition.y = uintBitsToFloat(attributeBuffer_buf[index_c + 20u]);
        targetPosition.z = uintBitsToFloat(attributeBuffer_buf[index_c + 21u]);
        float deltatime = builtiParametersValue.delta;

        age = age + builtiParametersValue.delta;
        alive = Reap(age, lifetime);
    {

    float var_16 = externalParametersValue.Float_2;;
    float var_22 = externalParametersValue.Float_0;;
    vec3 var_9 = vec3(0.00,0.00,1.00);;
    vec3 var_13 = externalParametersValue.Vector3f_0.xyz;;
    float var_17 = externalParametersValue.Float_1;;
    float var_19 = VFX_PI2;
    float var_23 = externalParametersValue.Float_3;;
    vec3 var_15 = targetPosition;;
    float var_27 = var_22 * var_16;;
    float var_30 = var_22 / var_23;;
    vec3 var_36 = var_9 * var_27;;
    float var_43 = var_30 * var_19;;
    float var_51 = var_43 * var_17;;
    vec3 var_59 = Rotate3D(var_15, vec3(0, 0, 0), vec3(0, 1, 0), var_51);;
    vec3 var_64 = var_59 + var_13;;
    vec3 var_67 = var_64 + var_36;;
    SetPosition_1(position, vec4(var_67, 1.0));
}
{

    bool var_8 = externalParametersValue.Boolean_0;;
    SetAlive(alive, var_8);
}
{

    float var_23 = externalParametersValue.Float_3;;
    float var_22 = externalParametersValue.Float_0;;
    float var_30 = var_22 / var_23;;
    float var_39 = clamp(var_30, 0.00, 1.00);;
    vec4 var_49 = texture(Gradient, vec2(var_39, 0.5));;
    float var_54 = var_49.x;
    SetAlpha_1(alpha, var_54);
}
{

    TriggerEventRate(rateCount, eventCount, velocity, externalParametersValue.Rate, builtiParametersValue.delta);
}
                {
                    uint i = 0u;
                    uint maxEvent = eventListOut_Counter0_Buf[1];
                    while(i < eventCount){
                        uint eventIdx = atomicAdd(eventListOut_Counter0_Buf[0], 1u);
                        if(eventIdx < maxEvent)
                        {
                            eventListOut0_buf[eventIdx] = particleIdx;
                        }
                        i = i + 0x1u;
                    }
                }
                eventCount = 0u;

                position = position + velocity * builtiParametersValue.delta;
                if(alive == 0u)
    {
        uint deadIndex = atomicAdd(deadList_andCounter_Buf[0], 1u) + 1u;
        deadList_andCounter_Buf[deadIndex] = particleIdx;
    }else
    {
        attributeBuffer_buf[index_c + 10u] = floatBitsToUint(lifetime);
        attributeBuffer_buf[index_c + 11u] = floatBitsToUint(age);
        attributeBuffer_buf[index_c] = floatBitsToUint(position.x);
        attributeBuffer_buf[index_c + 1u] = floatBitsToUint(position.y);
        attributeBuffer_buf[index_c + 2u] = floatBitsToUint(position.z);
        attributeBuffer_buf[index_c + 4u] = floatBitsToUint(velocity.x);
        attributeBuffer_buf[index_c + 5u] = floatBitsToUint(velocity.y);
        attributeBuffer_buf[index_c + 6u] = floatBitsToUint(velocity.z);
        attributeBuffer_buf[index_c + 7u] = floatBitsToUint(direction.x);
        attributeBuffer_buf[index_c + 8u] = floatBitsToUint(direction.y);
        attributeBuffer_buf[index_c + 9u] = floatBitsToUint(direction.z);
        attributeBuffer_buf[index_c + 44u] = inWorldSpace;
    attributeBuffer_buf[index_c + 17u] = floatBitsToUint(alpha);
        attributeBuffer_buf[index_c + 26u] = floatBitsToUint(rateCount);
        attributeBuffer_buf[index_c + 25u] = eventCount;
        attributeBuffer_buf[index_c + 19u] = floatBitsToUint(targetPosition.x);
        attributeBuffer_buf[index_c + 20u] = floatBitsToUint(targetPosition.y);
        attributeBuffer_buf[index_c + 21u] = floatBitsToUint(targetPosition.z);
        }

  }
  attributeBuffer_buf[index_c + 12u] = alive;
              }
return;
}
